@@ -1,5 +1,43 @@
- read.dta <- function(file, convert.dates=TRUE,tz="GMT",convert.factors=TRUE){
+
+read.dta <- function(file, convert.dates=TRUE,tz="GMT",
+                      convert.factors=TRUE,missing.type=FALSE){
     rval<-.External("do_readStata", file, PACKAGE = "foreign")
+
+
+    types<-attr(rval,"types")    
+    stata.na<-data.frame(type=251:255,
+                         min=c(101, 32741, 2147483621, 2^127, 2^1023),
+                         inc=c(1,1,1,2^115,2^1011)
+                         )
+
+         
+    if(!missing.type){
+        if (abs(attr(rval,"version"))==8){
+            for(v in which(types>250)){
+                this.type<-types[v]-250
+                rval[,v][rval[,v]>=stata.na$min[this.type]]<-NA
+            }
+        }
+    } else {
+        if (abs(attr(rval,"version"))==8){
+            missings<-vector("list",NCOL(rval))
+            names(missings)<-names(rval)
+            for(v in which(types>250)){
+                this.type<-types[v]-250
+                nas<-is.na(rval[,v]) |  rval[,v]>=stata.na$min[this.type]
+                natype<-(rval[nas,v]-stata.na$min[this.type])/stata.na$inc[this.type]
+                natype[is.na(natype)]<-0
+                missings[[v]]<-rep(NA,NROW(rval))
+                missings[[v]][nas]<-natype
+                rval[nas,v]<-NA
+            }
+            attr(rval,"missing")<-missings
+        } else {
+            warning("`missing.type' only applicable to version 8 files")
+        }
+        
+    }
+
     if (convert.dates){
         ff<-attr(rval,"formats")
         dates<-grep("%-*d",ff)
@@ -13,20 +51,33 @@
         for(v in factors)
             rval[[v]]<-factor(rval[[v]],levels=tt[[ll[v]]],labels=names(tt[[ll[v]]]))
     }
-        
+
+
     rval
+    
 }
 
 write.dta <- function(dataframe, file, version = 6,convert.dates=TRUE,tz="GMT",
                       convert.factors=c("labels","string","numeric","codes"))
 {
+
+    if (version<6) stop("Version must be 6-8")
+    if (version>8) {
+        warning("Version must be 6-8: using 7")
+        version<-7
+    }
+        
+
+    
     namelength<-if (version==6) 8 else 31
-    nn<-abbreviate(names(dataframe),namelength )
+    oldn<-names(dataframe)
+    nn<-abbreviate(oldn,namelength )
     if (any(nchar(nn)>namelength))
         stop("Can't uniquely abbreviate variable names")
-    if (any(nchar(names(dataframe))>namelength))
+    if (any(nchar(oldn)>namelength))
         warning("Abbreviating variable names")
     names(dataframe)<-nn
+    attr(dataframe,"orig.names")<-oldn
     
     if (convert.dates){
         dates<-which(isdate<-sapply(dataframe,function(x) inherits(x,"POSIXt")))
@@ -57,6 +108,6 @@ write.dta <- function(dataframe, file, version = 6,convert.dates=TRUE,tz="GMT",
      
     if (any(sapply(dataframe, function(x) !is.null(dim(x)))))
         stop("Can't handle multicolumn columns")
-    invisible(.External("do_writeStata", file, dataframe, version, leveltable,
+    invisible(.External("do_writeStata", file, dataframe, version, leveltable, 
                         PACKAGE = "foreign"))
 }
